#include "framecpp/config.h"

#include "framecpp/Common/Description.hh"
#include "framecpp/Common/IOStream.hh"

#include "framecpp/Version4/FrSerData.hh"
#include "framecpp/Version4/FrSE.hh"
#include "framecpp/Version4/FrSH.hh"
#include "framecpp/Version4/PTR_STRUCT.hh"

using namespace FrameCPP::Version_4;
using FrameCPP::Common::Description;
using FrameCPP::Common::FrameSpec;
using FrameCPP::Common::IStream;
using FrameCPP::Common::OStream;

//=======================================================================
// Static
//=======================================================================

static const FrameSpec::Info::frame_object_types s_object_id
= FrameSpec::Info::FSI_FR_SER_DATA;

//=======================================================================
// FrSerData::fr_ser_data_data_type
//=======================================================================

bool FrSerData::fr_ser_data_data_type::
operator==( const fr_ser_data_data_type& RHS ) const
{
#define CMP__(X) ( X == RHS.X )

  return ( ( &RHS == this )
	   || ( CMP__( name )
		&& CMP__( time )
		&& CMP__( sampleRate )
		&& CMP__( data )
		&& CMP__( serial )
		&& CMP__( table )
		) )
    ;
#undef CMP__
}

static const INT_2U MAX_REF = 2;

//=======================================================================
// FrSerData
//=======================================================================
FrSerData::
FrSerData( )
  : FrameSpec::Object( s_object_id, StructDescription( ) )
{
}

FrSerData::
FrSerData( const FrSerData& SerData )
  : FrameSpec::Object( s_object_id, StructDescription( ) )
{
  m_data.name = SerData.m_data.name;
  m_data.time = SerData.m_data.time;
  m_data.sampleRate = SerData.m_data.sampleRate;
  m_data.data = SerData.m_data.data;
  m_data.serial = SerData.m_data.serial;
  m_data.table = SerData.m_data.table;
}

FrSerData::
FrSerData( const std::string& name, const GPSTime& time, REAL_4 sampleRate )
  : FrameSpec::Object( s_object_id, StructDescription( ) )
{
  m_data.name = name;
  m_data.time = time;
  m_data.sampleRate = sampleRate;
}


FrSerData::
FrSerData( Previous::FrSerData& Source, Common::IStream* Stream )
  : FrameSpec::Object( s_object_id, StructDescription( ) )
{
  m_data.name = Source.GetName( );
  m_data.time = Source.GetTime( );
  m_data.sampleRate = Source.GetSampleRate( );
  m_data.data = Source.GetData( );
  if ( Stream )
  {
    //-------------------------------------------------------------------
    // Modify references
    //-------------------------------------------------------------------
    Stream->ReplaceRef( RefSerial( ), &(Source.RefSerial( )), MAX_REF );
    Stream->RemoveResolver( &(Source.RefMore( )), MAX_REF );
  }
}

FrSerData::
FrSerData( IStream& Stream )
  : FrameSpec::Object( s_object_id, StructDescription( ) )
{
      Stream >> m_data.name
	     >> m_data.time
	     >> m_data.sampleRate
	     >> m_data.data
	     >> m_data.serial
	     >> m_data.table
	;
      Stream.Next( this );
}

FrameCPP::cmn_streamsize_type FrSerData::
Bytes( const Common::StreamBase& Stream ) const
{
  return
    m_data.name.Bytes( )
    + sizeof( INT_4U )		// timeSec
    + sizeof( INT_4U )		// timeNsec
    + sizeof( m_data.sampleRate )
    + m_data.data.Bytes( )
    + Stream.PtrStructBytes( )	// serial
    + Stream.PtrStructBytes( )	// table
    + Stream.PtrStructBytes( )	// next
    ;
}

FrSerData* FrSerData::
Create( IStream& Stream ) const
{
  return new FrSerData( Stream );
}

const char* const FrSerData::
ObjectStructName( ) const
{
  return StructName( );
}

const Description* FrSerData::
StructDescription( )
{
  static Description ret;

  if ( ret.size( ) == 0 )
  {
    ret( FrSH( FrSerData::StructName( ), s_object_id,
	       "Serial Data Structure" ) );

    ret( FrSE( "name", "STRING" ) );
    ret( FrSE( "timeSec", "INT_4U" ) );
    ret( FrSE( "timeNsec", "INT_4U" ) );
    ret( FrSE( "sampleRate", "REAL_4" ) );
    ret( FrSE( "data", "STRING" ) );

    ret( FrSE( "serial", PTR_STRUCT::Desc( FrVect::StructName( ) ) ) );
    ret( FrSE( "table", PTR_STRUCT::Desc( FrTable::StructName( ) ) ) );

    ret( FrSE( "next", PTR_STRUCT::Desc( FrSerData::StructName( ) ) ) );
  }

  return &ret;
}

void FrSerData::
Write( OStream& Stream ) const
{
  Stream << m_data.name
	 << m_data.time
	 << m_data.sampleRate
	 << m_data.data
	 << m_data.serial
	 << m_data.table
    ;
  WriteNext( Stream );
}

bool FrSerData::
operator==( const Common::FrameSpec::Object& Obj ) const
{
  return ( *this == Obj );
}

const std::string& FrSerData::
GetName() const
{ 
   return m_data.name;
}


FrSerData& FrSerData::
Merge( const FrSerData& RHS )
{
  //:TODO: Need to implement Merge routine
  std::string msg( "Merge currently not implemented for " );
  msg += StructName( );

  throw std::domain_error( msg );
  return *this;
}


void FrSerData::
TOCQuery( int InfoClass, ... ) const
{
  using Common::TOCInfo;

  va_list	vl;
  va_start( vl, InfoClass );
  while ( InfoClass != TOCInfo::IC_EOQ )
  {
    int data_type = va_arg( vl, int );
    switch( data_type )
    {
    case TOCInfo::DT_STRING_2:
      {
	STRING* data = va_arg( vl, STRING* );
	switch( InfoClass )
	{
	case TOCInfo::IC_NAME:
	  *data = GetName( );
	  break;
	default:
	  goto cleanup;
	  break;
	}
      }
      break;
    default:
      // Stop processing
      goto cleanup;
    }
    InfoClass = va_arg( vl, int );
  }
 cleanup:
  va_end( vl );
}

std::auto_ptr< FrameSpec::Object > FrSerData::
demote( INT_2U Target, IStream* Stream ) const
{
  if ( Target >= DATA_FORMAT_VERSION )
  {
    return std::auto_ptr< FrameSpec::Object >
      ( const_cast< FrSerData* >( this ) );
  }
  try
  {
    //-------------------------------------------------------------------
    // Copy non-reference information
    //-------------------------------------------------------------------
    // Do actual down conversion
    std::auto_ptr< Previous::FrSerData >
      retval( new Previous::FrSerData( GetName( ),
				       GetTime( ),
				       GetSampleRate( )
				       ) )
      ;
    retval->SetData( GetData( ) );
    if ( Stream )
    {
      //-----------------------------------------------------------------
      // Modify references
      //-----------------------------------------------------------------
      Stream->ReplaceRef( retval->RefSerial( ), &(RefSerial( )), MAX_REF );
      Stream->RemoveResolver( &(RefTable( )), MAX_REF );
    }
    //-------------------------------------------------------------------
    // Return demoted object
    //-------------------------------------------------------------------
    return std::auto_ptr< FrameSpec::Object >( retval.release( ) );
  }
  catch( ... )
  {
  }
  throw
    Unimplemented( "Object* FrSerData::demote( Object* Obj ) const",
		   DATA_FORMAT_VERSION, __FILE__, __LINE__ );
}

std::auto_ptr< FrameSpec::Object > FrSerData::
promote( INT_2U Target, Object* Obj, IStream* Stream ) const
{
  return Promote( Target, Obj, Stream );
}
