#ifndef FrameCPP_VERSION_4_FrTable_HH
#define FrameCPP_VERSION_4_FrTable_HH

#include <vector>

#include "framecpp/Version4/FrameSpec.hh"

#include "framecpp/Common/SearchContainer.hh"

#include "framecpp/Version4/Dimension.hh"
#include "framecpp/Version4/FrVect.hh"
#include "framecpp/Version4/GPSTime.hh"
#include "framecpp/Version4/STRING.hh"

namespace FrameCPP
{
  namespace Version_4
  {
    //===================================================================
    //
    //===================================================================
    class FrTableData
    {
    public:
      typedef std::vector<STRING> column_name_container_type;
      typedef INT_4U nRow_type;

      FrTableData( );

      FrTableData( const FrTableData& Source );

      void AppendComment( const std::string& comment );

      const column_name_container_type& GetColumnName( ) const;

      const std::string& GetComment( ) const;

      const std::string& GetName( ) const;
   
      nRow_type GetNRow( ) const;

    protected:
      struct fr_table_data_type {
	fr_table_data_type( );
	fr_table_data_type( const fr_table_data_type& Source );

	bool operator==( const fr_table_data_type& RHS ) const;

	cmn_streamsize_type Bytes( const Common::StreamBase& Stream ) const;

	STRING				name;
	STRING				comment;
	nRow_type			nRow;
	column_name_container_type	columnName;
      };

      fr_table_data_type	m_data;
    };

    inline FrTableData::fr_table_data_type::
    fr_table_data_type( )
      : name( "" ),
	comment( "" ),
	nRow( 0 )
    {
    }

    inline FrTableData::fr_table_data_type::
    fr_table_data_type( const fr_table_data_type& Source )
      : name( Source.name ),
	comment( Source.comment ),
	nRow( Source.nRow ),
	columnName( Source.columnName )
    {
    }

    inline bool FrTableData::fr_table_data_type::
    operator==( const fr_table_data_type& RHS ) const
    {
      return ( ( name == RHS.name )
	       && ( comment == RHS.comment )
	       && ( nRow == RHS.nRow )
	       && ( columnName == RHS.columnName )
	       );

    }

    inline void FrTableData::
    AppendComment( const std::string& comment )
    {
      return FrameCPP::Common::AppendComment( m_data.comment, comment );
    }

    inline const FrTableData::column_name_container_type& FrTableData::
    GetColumnName( ) const
    {
      return m_data.columnName;
    }

    inline const std::string& FrTableData::
    GetComment( ) const
    {
      return m_data.comment;
    }   
   
    inline const std::string& FrTableData::
    GetName( ) const
    {
      return m_data.name;
    }

    inline FrTableData::nRow_type FrTableData::
    GetNRow( ) const
    {
      return m_data.nRow;
    }      

    //===================================================================
    //
    //===================================================================
    class FrTable
      : public Common::FrameSpec::Object,
	public FrTableData
    {
    public:
      typedef INT_2U nColumn_type;
      typedef Common::SearchContainer< FrVect, &FrVect::GetName >
      column_type;

      typedef column_type::iterator iterator;
      typedef column_type::const_iterator const_iterator;

      //: Default constructor
      FrTable( );
      FrTable( const FrTable& table );
      FrTable( const std::string& name, INT_4U nrows );

      FrTable( const std::string& Name,
	       const std::string& Comment,
	       INT_4U NRows,
	       std::vector< STRING > ColumnNames );

      FrTable( const FrTableData& Source );

      virtual ~FrTable( );

      virtual cmn_streamsize_type Bytes( const Common::StreamBase& Stream ) const;

      virtual FrTable* Create( Common::IStream& Stream ) const;

      virtual const char* const ObjectStructName( ) const;

      virtual void Write( Common::OStream& Stream ) const;

      const std::string& GetNameSlow( ) const;

      nColumn_type GetNColumn( ) const;

      /* Mutators */
      const column_type& RefColumn( ) const;
   
      column_type& RefColumn( );

      //: Merge with another FrAdcData
      FrTable& Merge( const FrTable& RHS );

      static std::auto_ptr< Object >
      Promote( INT_2U Source,
	       Object* Obj, Common::IStream* Stream );

      //: Concatination operator
      FrTable& operator+=( const FrTable& RHS );

      static const Common::FrameSpec::Info::frame_object_types CLASS_ID
      = Common::FrameSpec::Info::FSI_FR_TABLE;

      static const char* const StructName( );

      /// \brief comparison operator
      bool operator==( const FrTable& RHS ) const;

      /// \brief comparison operator
      virtual bool operator==( const Common::FrameSpec::Object& Obj ) const;

    protected:
      /// \brief Down grade an object
      virtual std::auto_ptr< Object >
      demote( INT_2U Target, Common::IStream* Stream ) const;

      virtual std::auto_ptr< Object >
      promote( INT_2U Source,
	       Object* Obj, Common::IStream* Stream ) const;

    private:
      using Common::FrameSpec::Object::Create;

      FrTable( Common::IStream& Stream );

      fr_table_data_type	m_data;
      column_type		m_column;
    };

    inline const char* const FrTable::
    StructName( )
    {
      static const char* const name = "FrTable";
      return name;
    }

    inline bool FrTable::
    operator==( const FrTable& RHS ) const
    {
      return ( m_data == RHS.m_data );
    }

    inline const FrTable::column_type& FrTable::
    RefColumn( ) const
    {
      return m_column;
    }
   
    inline FrTable::column_type& FrTable::
    RefColumn( )
    {
      return m_column;
    }

    inline FrTable::nColumn_type FrTable::
    GetNColumn( ) const
    {
      return RefColumn( ).size( );
    }

  } // namespace - Version_4
} // namespace FrameCPP
#endif /* FrameCPP_VERSION_4_FrTable_HH */

