#ifndef FrameCPP_VERSION_6_FrTable_HH
#define FrameCPP_VERSION_6_FrTable_HH

#include <vector>

#include "framecpp/Version6/FrameSpec.hh"

#include "framecpp/Common/SearchContainer.hh"

#include "framecpp/Version4/FrTable.hh"

#include "framecpp/Version6/Dimension.hh"
#include "framecpp/Version6/FrVect.hh"
#include "framecpp/Version6/GPSTime.hh"
#include "framecpp/Version6/STRING.hh"

namespace FrameCPP
{
  namespace Version_6
  {
    //===================================================================
    //
    //===================================================================
    using Previous::FrTableData;

    class FrTable
      : public Common::FrameSpec::Object,
	public FrTableData
    {
    public:
      typedef INT_2U nColumn_type;
      typedef Common::SearchContainer< FrVect, &FrVect::GetName >
      column_type;

      typedef column_type::iterator iterator;
      typedef column_type::const_iterator const_iterator;

      //: Default constructor
      FrTable( );

      FrTable( const FrTable& Source );

      FrTable( const FrTableData& Source );

      FrTable( const std::string& name, INT_4U nrows );

      FrTable( Previous::FrTable& Source, Common::IStream* Stream );

      virtual ~FrTable( );

      virtual cmn_streamsize_type Bytes( const Common::StreamBase& Stream ) const;

      virtual FrTable* Create( Common::IStream& Stream ) const;

      virtual const char* const ObjectStructName( ) const;

      virtual void Write( Common::OStream& Stream ) const;

      const std::string& GetNameSlow( ) const;

      nColumn_type GetNColumn( ) const;

      /* Mutators */
      const column_type& RefColumn( ) const;
   
      column_type& RefColumn( );

      //: Merge with another FrAdcData
      FrTable& Merge( const FrTable& RHS );

      //: Concatination operator
      FrTable& operator+=( const FrTable& RHS );

      static const Common::FrameSpec::Info::frame_object_types CLASS_ID
      = Common::FrameSpec::Info::FSI_FR_TABLE;

      static const char* const StructName( );

      /// \brief comparison operator
      bool operator==( const FrTable& RHS ) const;

      /// \brief comparison operator
      virtual bool operator==( const Common::FrameSpec::Object& Obj ) const;

    protected:
      /// \brief Down grade an object
      virtual std::auto_ptr< Object >
      demote( INT_2U Target, Common::IStream* Stream ) const;

      /// \brief Upgrade an object
      virtual std::auto_ptr< Object >
      promote( INT_2U Target, Object* Obj, Common::IStream* Stream ) const;

    private:
      using Common::FrameSpec::Object::Create;

      FrTable( Common::IStream& Stream );

      fr_table_data_type	m_data;
      column_type		m_column;
    };

    inline const char* const FrTable::
    StructName( )
    {
      static const char* const name = "FrTable";
      return name;
    }

    inline bool FrTable::
    operator==( const FrTable& RHS ) const
    {
      return ( m_data == RHS.m_data );
    }

    inline const FrTable::column_type& FrTable::
    RefColumn( ) const
    {
      return m_column;
    }
   
    inline FrTable::column_type& FrTable::
    RefColumn( )
    {
      return m_column;
    }

    inline FrTable::nColumn_type FrTable::
    GetNColumn( ) const
    {
      return RefColumn( ).size( );
    }

  } // namespace - Version_6
} // namespace FrameCPP
#endif /* FrameCPP_VERSION_6_FrTable_HH */

