#include "general/config.h"

#include <sys/types.h>
#include <errno.h>
#include <unistd.h>

#include <string.h>
#include <signal.h>

#include <iostream>
#include <stdexcept>

#include "general/Fork.hh"

namespace General
{
  Fork::
  Fork( )
  {
    // Initialize the file descriptors that will be used for storing pipe
    //   information.
    for ( int x = 0; x < 2; x++ )
    {
      for ( int y = 0; y < 2; y++ )
      {
	m_pipes[x][y] = -1;
      }
    }
  }

  Fork::
  ~Fork( )
  {
    close_pipes( );
    if ( isParent( ) )
    {
      kill( m_child_pid, SIGKILL );
    }
  }

  void Fork::
  Spawn( )
  {
    try
    {
      //-----------------------------------------------------------------
      // Create a pipe
      //-----------------------------------------------------------------
      if ( ( pipe( m_pipes[ 0 ] ) < 0 )
	   || ( pipe( m_pipes[ 1 ] ) < 0 ) )
      {
	throw std::runtime_error( "Unable to create pipes" );
      }
      //-----------------------------------------------------------------
      // Create child
      //-----------------------------------------------------------------
      m_child_pid = fork( );
      switch( m_child_pid )
      {
      case -1:
	// Error condition
	break;
      case 0:
	// Child process
	close_pipe( 1, 1 );
	close_pipe( 0, 0 );
	evalChild( );
	break;
      default:
	// Parent process
	close_pipe( 1, 0 );
	close_pipe( 0, 1 );
	evalParent( );
	break;
      }
    }
    catch( ... )
    {
      close_pipes( );
      throw;	// Rethrow the execption
    }
  }

  void Fork::
  close_pipes( )
  {
    for ( int x = 0; x < 2; x++ )
    {
      for ( int y = 0; y < 2; y++ )
      {
	if ( m_pipes[x][y] >= 0 )
	{
	  close_pipe( x, y );
	}
      }
    }
  }

  void Fork::
  close_pipe( int Major, int Minor )
  {
    close( m_pipes[ Major ][ Minor ] );
    m_pipes[ Major ][ Minor ] = -1;
  }
}
